<?php

namespace App\Http\Resources;

use App\Models\AccountReceivable;
use App\Models\SupplierContact;
use App\Models\ClientAddress;
use App\Models\ClientSetting;
use App\Models\Cartola;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class SupplierResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @return array<string, mixed>
     */
    public function toArray(Request $request): array
    {
        $wayToPay = $this->relationLoaded('wayToPay') ? $this->whenLoaded('wayToPay') : null;
        $segment = $this->relationLoaded('segment') ? $this->whenLoaded('segment') : null;
        $country = $this->relationLoaded('country') ? $this->whenLoaded('country') : null;
        $department = $this->relationLoaded('department') ? $this->whenLoaded('department') : null;
        $province = $this->relationLoaded('province') ? $this->whenLoaded('province') : null;
        $district = $this->relationLoaded('district') ? $this->whenLoaded('district') : null;
        $company = $this->relationLoaded('company') ? $this->whenLoaded('company') : null;
        $addresses = $this->relationLoaded('addresses') ? $this->whenLoaded('addresses') : null;
        $salesman = $this->relationLoaded('salesman') ? $this->whenLoaded('salesman') : null;
        $businessArea = $this->relationLoaded('businessArea') ? $this->whenLoaded('businessArea') : null;

        $contacts = SupplierContact::where('idsupplier', $this->id)->active()->get() ?? [];
        $clientSetting = ClientSetting::where('idclient', $this->id)->active()->first() ?? null;

        return [
            'id' => $this->id,
            'rut' => $this->rut,
            'name_rz' => $this->name_rz,
            'fantasy_name' => $this->fantasy_name,
            'web' => $this->web,
            'giro' => $this->giro,
            'email' => $this->email,
            'phone' => $this->phone,
            'fax' => $this->fax,
            'type' => $this->type,
            'legal_nature' => $this->legal_nature,
            'idwaytopay' => $this->idwaytopay,
            $this->mergeWhen($wayToPay, fn() => [
                'waytopay' => $wayToPay->name
            ]),
            'idsalesman' => $this->idsalesman,
            $this->mergeWhen($salesman, fn() => [
                'salesman' => $salesman->name . ' ' . $salesman->surname,
                'salesman_rut' => $salesman->rut,
            ]),
            'idbussines_area' => $this->idbussines_area,
            $this->mergeWhen($businessArea, fn() => [
                'iderp_business_area' => $businessArea->iderp,
                'business_area' => $businessArea->name,
            ]),
            $this->mergeWhen($segment, fn() => [
                'idsegment' => $segment->id,
                'segment' => $segment->name
            ]),
            $this->mergeWhen($country, fn() => [
                'idcountry' => $country->id,
                'country' => $country->name
            ]),
            $this->mergeWhen($department, fn() => [
                'iddepartment' => $department->id,
                'department' => $department->name
            ]),
            $this->mergeWhen($province, fn() => [
                'idprovince' => $province->id,
                'province' => $province->name
            ]),
            $this->mergeWhen($district, fn() => [
                'iddistrict' => $district->id,
                'district' => $district->name
            ]),
            'address' => $this->address,
            $this->mergeWhen($company, fn() => [
                'idcompany' => $company->id,
                'company' => $company->business_name,
                'rut_company' => $company->rut,
                'cartola_config' => $company->companyConfiguration->cartola
            ]),
            'contacts' => $contacts,
            'clientAddresses' => ClientAddressResource::collection($addresses),
            'clientSetting' => $clientSetting,
            'status' => $this->status,
            'balance_infavor' => optional(Cartola::where('idsupplier', $this->id)
                ->active()
                ->where('type', 6)
                ->orderBy('id', 'desc')
                ->first(), function ($lastCartola) {
                    $hasType3or4 = Cartola::where('idsupplier', $this->id)
                        ->active()
                        ->whereIn('type', [3]) // pagos
                        ->where('id', '>', $lastCartola->id) // Solo registros más recientes
                        ->exists();

                    return !$hasType3or4 ? -($lastCartola->balance) : 0;
                }),
            $this->mergeWhen(request()->boolean('accountReceivable'), function () {

                $accountsReceivableExpired = AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $this->id)->active()
                )
                    ->where('deadline', '<', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->count();

                return [
                    'accounts_receivable' => AccountReceivable::whereHas(
                        'ticket',
                        fn($q) => $q->where('idcustomer', $this->id)->active()
                    )
                        ->where('paid', 0)
                        ->active()
                        ->count(),

                    'amount_receivable' => (float) AccountReceivable::whereHas(
                        'ticket',
                        fn($q) => $q->where('idcustomer', $this->id)->active()
                    )
                        ->where('paid', 0)
                        ->active()
                        ->sum('debt'),

                    'accounts_receivable_expired' => $accountsReceivableExpired,

                    'amount_receivable_expired' => (float) AccountReceivable::whereHas(
                        'ticket',
                        fn($q) => $q->where('idcustomer', $this->id)->active()
                    )
                        ->where('deadline', '<', Carbon::now()->toDateString())
                        ->where('paid', 0)
                        ->active()
                        ->sum('debt'),

                    'accounts_receivable_not_expired' => AccountReceivable::whereHas(
                        'ticket',
                        fn($q) => $q->where('idcustomer', $this->id)->active()
                    )
                        ->where('deadline', '>=', Carbon::now()->toDateString())
                        ->where('paid', 0)
                        ->active()
                        ->count(),

                    'amount_receivable_not_expired' => (float) AccountReceivable::whereHas(
                        'ticket',
                        fn($q) => $q->where('idcustomer', $this->id)->active()
                    )
                        ->where('deadline', '>=', Carbon::now()->toDateString())
                        ->where('paid', 0)
                        ->active()
                        ->sum('debt'),

                    'row_color' => $this->getRowColor($accountsReceivableExpired)
                ];
            }),
        ];
    }

    private function getRowColor($accounts_receivable_expired)
    {
        if ($accounts_receivable_expired > 0) {
            return '#FFDAB9';
        } else {
            return 'inherit';
        }
    }
}
