<?php

namespace App\Imports;

use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use App\Models\Supplier;
use App\Models\ClientAddress;
use App\Models\District;

class ClientsAddressDataImport implements ToCollection, WithHeadingRow
{

    public function collection(Collection $clients)
    {
        $type = "1";
        $idcompany = request('idcompany');

        // Obtener clientes existentes previamente
        $existingClients = Supplier::where('idcompany', $idcompany)
            ->active()
            ->pluck('id', 'name_rz');

        // Obtener todas las direcciones existentes asociadas a clientes y normalizarlas
        $existingAddresses = ClientAddress::where('idcompany', $idcompany)
            ->get(['idclient', 'address'])
            ->map(function ($address) {
                return [
                    'idclient' => $address->idclient,
                    'address' => $this->normalizeString($address->address),
                ];
            });

        // Obtener distritos con su jerarquía (en memoria)
        $districts = District::with(['province.department.country'])
            ->get()
            ->keyBy(fn($item) => $this->normalizeString($item->name));

        $dataClients = [];
        $dataAddresses = [];

        foreach ($clients as $client) {
            $normalizedClientName = $this->normalizeString($client['razon_soc']);
            $normalizedAddress = $this->normalizeString($client['direccion'] ?? '');

            // Validar si el cliente ya existe
            $clientId = $existingClients[$normalizedClientName] ?? null;
            if (!$clientId) {
                $dataClients[] = [
                    'name_rz' => $client['razon_soc'],
                    'idcompany' => $idcompany,
                ];
            }

            // Procesar direcciones
            $comuna_normalizada = $this->normalizeString($client['comuna']);
            $district = $districts->get($comuna_normalizada);
            $province = $district->province ?? null;
            $department = $province->department ?? null;
            $country = $department->country ?? null;

            // Validar si la dirección ya existe para este cliente
            $isDuplicate = $existingAddresses->contains(function ($existing) use ($normalizedAddress) {
                return $existing['address'] === $normalizedAddress;
            });

            if (!$isDuplicate) {
                $dataAddresses[] = [
                    'name_rz' => $client['razon_soc'],
                    'address' => $client['direccion'] ?? null,
                    'idcountry' => $country->id ?? null,
                    'idprovince' => $province->id ?? null,
                    'iddistrict' => $district->id ?? null,
                    'idcompany' => $idcompany,
                    'created_at' => now(),
                    'updated_at' => now(),
                ];
            }
        }

        // Insertar clientes nuevos
        if (!empty($dataClients)) {

            // Actualizar la lista de clientes existentes
            $existingClients = Supplier::where('idcompany', $idcompany)
                ->pluck('id', 'name_rz');
        }

        // Mapear IDs de clientes para direcciones
        foreach ($dataAddresses as &$address) {
            $address['idclient'] = $existingClients[$address['name_rz']] ?? null;
            unset($address['name_rz']);
        }

        // Insertar direcciones
        if (!empty($dataAddresses)) {
            ClientAddress::insert($dataAddresses);
        }
    }

    private function normalizeString($string)
    {
        $string = trim(mb_strtolower($string, 'UTF-8'));
        $string = preg_replace('/\s+/', ' ', $string);
        $string = str_replace(
            ['á', 'é', 'í', 'ó', 'ú', 'ñ'], 
            ['a', 'e', 'i', 'o', 'u', 'n'], 
            $string
        );
        return $string;
    }

}
