<?php

namespace App\Services;

use App\Exports\AccountReceivableExport;
use App\Http\Resources\AccountReceivableResource;
use App\Http\Resources\PaymentVoucherResource;
use App\Http\Resources\SupplierResource;
use App\Models\AccountReceivable;
use App\Models\AccountReceivableDet;
use App\Models\Cartola;
use App\Models\Company;
use App\Models\PaymentVoucher;
use App\Models\ShoppingCart;
use App\Models\StockProduct;
use App\Models\Supplier;
use App\Models\Ticket;
use App\Models\CompanyConfiguration;
use App\Traits\HasResponse;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Barryvdh\DomPDF\Facade\Pdf;
use Tymon\JWTAuth\Contracts\Providers\JWT;
use Tymon\JWTAuth\Facades\JWTAuth;
use Maatwebsite\Excel\Facades\Excel;

/**
 * Class AccountReceivableService
 * @package App\Services
 */
class AccountReceivableService
{
    use HasResponse;

    public function list($params, $withPagination)
    {
        try {
            $data = AccountReceivable::filters()->orderBy('id', 'asc');

            $data = !empty($withPagination)
                ? $data->paginate($withPagination['perPage'], page: $withPagination['page'])
                : $data->get();

            $paginationTotal = null;
            if (!empty($withPagination)) $paginationTotal = $data->total();

            $data = AccountReceivableResource::collection($data->load(
                'ticket.wayToPay',
                'ticket.typeDocument'
            ));

            return $this->successPaginationResponse('Lectura exitosa.', $paginationTotal, $data);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function register($params)
    {
        DB::beginTransaction();
        try {
            // $ticket = Ticket::where('id', $params['idticket'])
            //     ->update([
            //         'idway_to_pay' => $params['idway_to_pay'],
            //         'status' => 1
            //     ]);

            $ticket = Ticket::activeForId($params['idticket'])
                ->first()
                ->load(
                    'wayToPay'
                );

            // $ticket->update([
            //     'status_invoice'    => 1,
            //     'folio'             => rand(10000, 99999)
            // ]);

            ShoppingCart::where('id', $ticket->idshopping_cart)->update(['status_ticket' => 1]);

            # Calcular fecha límite
            if ($ticket['wayToPay']['SII_fmapago'] == '2') {
                # Crédito
                $deadline = Carbon::now()->addDays($ticket['wayToPay']['days'])->toDateString();
            } else {
                # Contado
                $deadline = Carbon::now()->toDateString();
            }

            $account = AccountReceivable::create([
                'idticket'      => $params['idticket'],
                'deadline'      => $deadline,
                'total'         => $ticket->total,
                'amount_paid'   => 0,
                'debt'          => $ticket->total
            ]);

            # Ultimo registro de datos del cliente
            $lastDetail = Cartola::where('idsupplier', $ticket->idcustomer)
                ->company()
                ->active()
                ->orderBy('id', 'desc')
                ->first();

            $lastBalance = $lastDetail->balance ?? 0;

            # Registrar en cartola que la factura fue emitida
            Cartola::create([
                'correlative' => Cartola::where('idcompany', $ticket->idcompany)->count() + 1,
                'idsupplier' => $ticket->idcustomer,
                'idticket' => $ticket->id,
                'type' => 1,
                'charge' => $ticket->total,
                'payment' => 0,
                'balance' => $lastBalance + $ticket->total,
                'gloss' => 'Venta de productos - ' . $ticket->folio,
                'idcompany' => $ticket->idcompany,
                'iduser' => JWTAuth::user()->id,
            ]);

            DB::commit();
            return $this->successResponse('Cuenta por cobrar registrada con éxito.', $account);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la creación de cuenta por cobrar.', $th->getMessage());
        }
    }

    public function pay($params)
    {
        DB::beginTransaction();
        try {
            # Validar parámetros
            $validation = Validator::make($params, [
                'account_receivable'                 => ['required', 'array', 'min:1'],
                'account_receivable.*.id'            => ['required_with:account_receivable', 'integer', 'validate_ids_exist:AccountReceivable'],
                'account_receivable.*.amount_pay'    => ['required_with:account_receivable', 'integer', 'min:1'],

                'payment_voucher'                    => ['required', 'array', 'min:1'],
                'payment_voucher.*.idsupplier'       => ['required_with:payment_voucher', 'integer', 'validate_ids_exist:Supplier'],
                'payment_voucher.*.idway_to_pay'     => ['required_with:payment_voucher', 'integer', 'validate_ids_exist:WayToPay'],
                'payment_voucher.*.idsource_bank'    => ['nullable', 'integer', 'validate_ids_exist:Bank'],
                'payment_voucher.*.idreceiving_bank' => ['nullable', 'integer', 'validate_ids_exist:Bank'],
                'payment_voucher.*.series'           => ['nullable', 'string'],
                'payment_voucher.*.document_date'    => ['nullable', 'string'],
                'payment_voucher.*.amount'           => ['required_with:payment_voucher', 'integer', 'min:1'],

                'idcompany'                          => ['required', 'integer', 'validate_ids_exist:Company'],
            ]);

            if ($validation->fails()) return $this->errorResponse($validation->errors(), 400);

            # Extraer los IDs de las cuentas seleccionadas y sus montos
            $accountsData = collect($params['account_receivable']);

            # Total pagado (ingresado en comprobantes)
            $totalAmountPay = collect($params['payment_voucher'])->sum('amount');

            # Total a pagar (ingresado por el usuario)
            $totalToPay = $accountsData->sum('amount_pay');

            if ($totalToPay > $totalAmountPay) {
                return $this->errorResponse('Los montos ingresados en los comprobantes no son mayores o iguales al monto total a pagar.', 400);
            }

            # Series de los comprobantes
            $series = collect($params['payment_voucher'])->pluck('series')->toArray();

            # Validar serie del comprobante de pago
            $paymentVoucher = PaymentVoucher::whereIn('series', $series)
                ->active()
                ->company()
                ->first();
            if ($paymentVoucher) return $this->errorResponse('La serie ya se encuentra registrada.', 400);

            # Procesar cada comprobante de pago
            foreach ($params['payment_voucher'] as $voucher) {
                # Crear correlativo
                $n_voucher = PaymentVoucher::where('idcompany', $params['idcompany'])->count() + 1;
                $correlative = date('Ymd') . '-' . str_pad($n_voucher, 4, '0', STR_PAD_LEFT);

                # Registrar pago
                $paymentVoucher = PaymentVoucher::create([
                    'correlative'      => $correlative,
                    'idcompany'        => $params['idcompany'],
                    'idsupplier'       => $voucher['idsupplier'],
                    'idway_to_pay'     => $voucher['idway_to_pay'],
                    'idsource_bank'    => $voucher['idsource_bank'] ?? null,
                    'idreceiving_bank' => $voucher['idreceiving_bank'] ?? null,
                    'series'           => $voucher['series'],
                    'entry_date'       => Carbon::now()->toDateString(),
                    'document_date'    => $voucher['document_date'] ?? null,
                    'amount'           => $voucher['amount'],
                ]);

                # Saldo a favor que se distribuira entre el pago de las cuentas
                $remainingAmount = $paymentVoucher->amount;

                # Monto distribuido
                $amountToPayCartola = 0;

                # Recorrer las cuentas por cobrar para aplicar el pago
                foreach ($accountsData as $accountData) {
                    $account = AccountReceivable::where('id', $accountData['id'])
                        ->where('paid', 0)
                        ->active()
                        ->first()
                        ->load('ticket');

                    # Si la cuenta no existe o ya está pagada, saltar
                    if (!$account) continue;

                    # Determinar cuánto se puede pagar de esta cuenta (según el monto ingresado)
                    $amountToPay = min($remainingAmount, $account->debt, $accountData['amount_pay']);
                    $amountToPayCartola += $amountToPay;

                    # Actualizar la deuda de la cuenta
                    $account->update([
                        'debt'         => $account->debt - $amountToPay,
                        'amount_paid'  => $account->amount_paid + $amountToPay,
                        'paid'         => $account->debt - $amountToPay == 0 ? 1 : 0,
                        'payment_date' => $account->debt - $amountToPay == 0 ? Carbon::now()->toDateTimeString() : null
                    ]);
                    
  

                    # Registrar detalle en la cuenta por cobrar
                    AccountReceivableDet::create([
                        'idaccount_receivable' => $account->id,
                        'idpayment_voucher'    => $paymentVoucher->id,
                        'amount'               => $amountToPay,
                        'idcompany'            => $params['idcompany'],
                    ]);

                    # Crear comprobante
                    if ($account->paid == 1) {
                        $this->createOrUpdatePDF($account->id);
                    }

                    # Actualizar el saldo restante a favor
                    $remainingAmount -= $amountToPay;

                    # Si ya no hay saldo restante, romper el ciclo
                    if ($remainingAmount <= 0) {
                        break;
                    }
                }

                # Último registro de cartola del cliente
                $lastDetail = Cartola::where('idsupplier', $voucher['idsupplier'])
                ->company()
                ->active()
                ->orderBy('id', 'desc')
                ->first();

                $lastBalance = $lastDetail->balance ?? 0;
                
                # Registrar en Cartola
                Cartola::create([
                    'correlative'       => Cartola::where('idcompany', $params['idcompany'])->count() + 1,
                    'idsupplier'        => $voucher['idsupplier'],
                    'idticket'          => null,
                    'idpayment_voucher' => $paymentVoucher->id,
                    'charge'            => 0,
                    'payment'           => $amountToPayCartola,
                    'balance'           => $lastBalance - $amountToPayCartola,
                    'idway_to_pay'      => $paymentVoucher->idway_to_pay,
                    'idcompany'         => $params['idcompany'],
                    'iduser'            => JWTAuth::user()->id,
                    'type'              => 3,
                    'gloss'             => 'Pago de factura varios'
                ]);

                # Obtener la configuración de la empresa
                $companyConfig = CompanyConfiguration::where('idcompany', $params['idcompany'])->first();
                $cartolaConfig = $companyConfig->cartola ?? 2; 

                # Determinar qué hacer con el saldo restante
                if ($remainingAmount > 0) {
                    if ($cartolaConfig == 2) {
                        // Caso 2: Saldos a favor -> Se registra en la cartola

                        $saldoInfavor = optional(Cartola::where('idsupplier', $voucher['idsupplier'])
                        ->active()
                        ->where('type', 6)
                        ->orderBy('id', 'desc')
                        ->first(), function ($lastCartola) use ($voucher) {
                            $hasType3or4 = Cartola::where('idsupplier', $voucher['idsupplier'])
                                ->active()
                                ->whereIn('type', [3]) // pagos
                                ->where('id', '>', $lastCartola->id) // Solo registros más recientes
                                ->exists();
        
                            return !$hasType3or4 ? -($lastCartola->balance) : 0;
                        });

                        Cartola::create([
                            'correlative'       => Cartola::where('idcompany', $params['idcompany'])->count() + 1,
                            'idsupplier'        => $voucher['idsupplier'],
                            'charge'            => 0,
                            'payment'           => $remainingAmount - $saldoInfavor,
                            'balance'           => 0 - ($remainingAmount - $saldoInfavor),
                            'idcompany'         => $params['idcompany'],
                            'iduser'            => JWTAuth::user()->id,
                            'type'              => 6, // Tipo 6: saldo a favor
                            'gloss'             => 'Saldo a favor registrado'
                        ]);
                    } else {
                        // Caso 1: Cartola de ajuste -> Eliminar saldo restante
                        $remainingAmount = 0;
                    }
                }

                # Sumar el saldo restante a favor
                $paymentVoucher->update(['remaining_amount' => $remainingAmount]);

                # Crear PDF
                $this->createOrUpdateVoucherPDF($paymentVoucher->id);
            }

            DB::commit();
            return $this->successResponse('Pago registrado con éxito.');
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante el registro de pago.', $th->getMessage());
        }
    }

    public function createOrUpdatePDF($idaccount)
    {
        try {
            # Obtener cuenta por pagar
            $account = AccountReceivable::activeForId($idaccount)->first();
            $idcompany = Ticket::where('id', $account->idticket)->value('idcompany');

            # Eliminar documento anterior
            if ($account->url_document && Storage::exists($account->url_document)) {
                Storage::delete($account->url_document);
            }

            # Generar pdf
            $data = $account->load(
                'ticket.wayToPay',
                'ticket.customer',
                'ticket.clientAddress',
                'accountReceivableDets.paymentVoucher.sourceBank',
                'accountReceivableDets.paymentVoucher.receivingBank',
                'accountReceivableDets.paymentVoucher.wayToPay',
            );

            $data = AccountReceivableResource::make($data);
            $data = json_decode(json_encode($data));
            $pdf  = Pdf::loadView('pdfs.accountPDF', compact('data'));

            # Generar ruta
            $company     = Company::activeForId($idcompany)->first();
            $companyName = str_replace(' ', '_', $company->business_name);
            $number      = rand(1000, 9999);
            $filename    = "account_$idaccount" . "_" . "$number.pdf";
            $path        = "public/companies/$companyName/accounts/$filename";

            # Guardar pdf en el storage
            Storage::put($path, $pdf->output());

            # Guardar url
            $url = Storage::url($path);
            AccountReceivable::where('id', $idaccount)
                ->update(['url_document' => $url]);

            return true;
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function downloadExcel($id)
    {
        try {
            # Validar existencia del cliente
            $supplier = Supplier::activeForID($id)->first();
            if (!$supplier) return $this->errorResponse('Cliente no válido.', 400);

            $summary = [
                'accounts_receivable' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('paid', 0)
                    ->active()
                    ->count(),

                'amount_receivable' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('paid', 0)
                    ->active()
                    ->sum('debt'),

                'accounts_receivable_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '<', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->count(),

                'amount_receivable_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '<', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->sum('debt'),

                'accounts_receivable_not_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '>=', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->count(),

                'amount_receivable_not_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '>=', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->sum('debt'),
            ];

            $accounts = AccountReceivable::whereHas('ticket', function ($q) use ($supplier) {
                $q->where('idcustomer', $supplier->id)->active();
            })
                ->where('paid', 0)
                ->active()
                ->orderBy('deadline', 'asc')
                ->get()
                ->load('ticket.wayToPay');

            # Para que el resource me retorne el color de las filas
            $accounts = AccountReceivableResource::collection($accounts);

            $data = [
                'name_rz'  => $supplier->name_rz,
                'rut'      => $supplier->rut,
                'summary'  => $summary,
                'accounts' => $accounts
            ];

            $name = str_replace(' ', '_', $supplier->name_rz);
            $fileName = "$name.xlsx";
            $baseUrl = url('/');

            Excel::store(new AccountReceivableExport($data), $fileName, 'public');

            return $this->successResponse('OK', $baseUrl . '/storage/' . $fileName);
        } catch (\Throwable $th) {
            return $this->externalError('durante la descarga.', $th->getMessage());
        }
    }

    public function downloadPDF($id)
    {
        try {
            $supplier = Supplier::activeForID($id)->first();
            if (!$supplier) return $this->errorResponse('Cliente no válido.', 400);

            $summary = [
                'accounts_receivable' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('paid', 0)
                    ->active()
                    ->count(),
                'amount_receivable' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('paid', 0)
                    ->active()
                    ->sum('debt'),
                'accounts_receivable_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '<', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->count(),
                'amount_receivable_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '<', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->sum('debt'),
                'accounts_receivable_not_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '>=', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->count(),
                'amount_receivable_not_expired' => (int) AccountReceivable::whereHas(
                    'ticket',
                    fn($q) => $q->where('idcustomer', $supplier->id)->active()
                )
                    ->where('deadline', '>=', Carbon::now()->toDateString())
                    ->where('paid', 0)
                    ->active()
                    ->sum('debt'),
            ];

            $accounts = AccountReceivable::whereHas('ticket', function ($q) use ($supplier) {
                $q->where('idcustomer', $supplier->id)->active();
            })
                ->where('paid', 0)
                ->active()
                ->orderBy('deadline', 'asc')
                ->get()
                ->load('ticket.wayToPay');

            $accounts = AccountReceivableResource::collection($accounts);

            // Usar $accounts en lugar de $data['accounts']
            $totalFactura = collect($accounts)->sum('total');
            $montoPagado = collect($accounts)->sum('amount_paid');
            $saldoPendiente = collect($accounts)->sum('debt');

            $totals = [
                'totalFactura' => $totalFactura,
                'montoPagado' => $montoPagado,
                'saldoPendiente' => $saldoPendiente
            ];

            $data = [
                'data' => [
                    'supplierName' => $supplier->name_rz,
                    'rut'          => $supplier->rut,
                    'summary'      => $summary,
                    'accounts'     => $accounts,
                    'totals'       => $totals
                ]
            ];


            $pdf = Pdf::loadView('pdfs.account_receivable', $data)
                ->setPaper('a4', 'landscape');

            $fileName = str_replace(' ', '_', $supplier->name_rz) . '.pdf';
            $filePath = storage_path('app/public/');
            if (!file_exists($filePath)) {
                mkdir($filePath, 0777, true);
            }

            $path = $filePath . $fileName;
            $pdf->save($path);

            $url = asset('storage/' . $fileName);

            return $this->successResponse('PDF generado con éxito.', $url);
        } catch (\Throwable $th) {
            return $this->externalError('durante la descarga del PDF.', $th->getMessage());
        }
    }

    public function createOrUpdateVoucherPDF($idpayment)
    {
        try {
            # Obtener cuenta por pagar
            $voucher = PaymentVoucher::activeForId($idpayment)->first();
            $idcompany = $voucher->idcompany;

            # Eliminar documento anterior
            if ($voucher->url_document && Storage::exists($voucher->url_document)) {
                Storage::delete($voucher->url_document);
            }

            # Generar pdf
            $data = $voucher->load(
                'supplier',
                'wayToPay',
                'accountReceivableDets.accountReceivable.ticket'
            );

            $data = PaymentVoucherResource::make($data);
            $data = json_decode(json_encode($data));
            $pdf  = Pdf::loadView('pdfs.paymentVoucherPDF', compact('data'));

            # Generar ruta
            $company     = Company::activeForId($idcompany)->first();
            $companyName = str_replace(' ', '_', $company->business_name);
            $number      = rand(1000, 9999);
            $filename    = "account_$idpayment" . "_" . "$number.pdf";
            $path        = "public/companies/$companyName/vouchers/$filename";

            # Guardar pdf en el storage
            Storage::put($path, $pdf->output());

            # Guardar url
            $url = Storage::url($path);
            PaymentVoucher::where('id', $idpayment)
                ->update(['url_document' => $url]);

            return true;
        } catch (\Throwable $th) {
            throw $th;
        }
    }
}
