<?php

namespace App\Services;

use App\Http\Resources\IntakeCartResource;
use App\Models\IntakeCart;
use App\Models\IntakeCartDetail;
use App\Traits\HasResponse;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\Facades\JWTAuth;

/**
 * Class IntakeCartService
 * @package App\Services
 */
class IntakeCartService
{
    use HasResponse;

    public function list($params, $withPagination)
    {
        try {
            $data = IntakeCart::filters();

            $data = !empty($withPagination)
                ? $data->paginate($withPagination['perPage'], page: $withPagination['page'])
                : $data->get();

            $paginationTotal = null;
            if (!empty($withPagination)) $paginationTotal = $data->total();

            $data = IntakeCartResource::collection($data->load(
                'supplier',
                'details.product.unitMeasure',
                'company'
            ));

            return $this->successPaginationResponse('Lectura exitosa.', $paginationTotal, $data);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function register($params)
    {
        DB::beginTransaction();
        try {
            # Verificar si existe el carrito de ingreso del proveedor
            $intakeCart = IntakeCart::when(
                isset($params['idsupplier']),
                fn($q) => $q->where('idsupplier', $params['idsupplier'])
            )->where('iduser_created', JWTAuth::user()->id)
                ->where('status_entry', 0)
                ->company()
                ->active()
                ->first();

            if (!$intakeCart) {
                $intakeCart = IntakeCart::create([
                    'iduser_created' => JWTAuth::user()->id,
                    'idsupplier'     => $params['idsupplier'] ?? null,
                    'idcompany'      => $params['idcompany']
                ]);
            };

            # Verificamos si el producto ya fue agregado
            $verifyProduct = IntakeCartDetail::where('idproduct', $params['idproduct'])
                ->where('idintake_cart', $intakeCart->id)
                ->active()
                ->first();

            # Si existe
            if ($verifyProduct) {
                # Agregar cantidad enviada por parámetro
                $quantity = $verifyProduct->quantity + $params['quantity'];

                $verifyProduct->update([
                    'quantity' => $quantity
                ]);
            } else {
                # Crear cantidad enviada por el parámetro o en su defecto con 1
                $verifyProduct = IntakeCartDetail::create([
                    'idintake_cart' => $intakeCart->id,
                    'idproduct' => $params['idproduct'],
                    'quantity'  => $params['quantity'] ?? 1,
                ]);
            }

            # Cargar relaciones
            $intakeCart = $intakeCart->load(
                'supplier',
                'details.product',
                'company'
            );
            $intakeCart = IntakeCartResource::make($intakeCart);

            DB::commit();
            return $this->successResponse('Producto agregado al carrito satisfactoriamente.', $intakeCart);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante el agregado del producto al carrito.', $th->getMessage());
        }
    }

    public function updateQuantity($id, $params)
    {
        DB::beginTransaction();
        try {
            # Validar existencia del registor
            $detail = IntakeCartDetail::activeForID($id)->first();
            if (!$detail) return $this->errorResponse('Registro no válido.', 400);

            $validator = Validator::make($params, [
                'quantity' => ['required', 'numeric', 'min:1']
            ]);
            if ($validator->fails()) return $this->errorResponse($validator->errors(), 400);

            $detail->update($params);

            $intakeCart = IntakeCart::activeForId($detail->idintake_cart)
                ->first()
                ->load(
                    'supplier',
                    'details.product',
                    'company'
                );
            $intakeCart = IntakeCartResource::make($intakeCart);

            DB::commit();
            return $this->successResponse('Cantidad actualizada con éxito.', $intakeCart);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la actualización de la cantidad.', $th->getMessage());
        }
    }

    public function deleteProduct($id)
    {
        DB::beginTransaction();
        try {
            # Validar existencia del registor
            $detail = IntakeCartDetail::activeForID($id)->first();
            if (!$detail) return $this->errorResponse('Registro no válido.', 400);

            # Eliminar
            $detail->update(['status' => 2]);

            # Validar si aún hay productos en el carrito de ingreso
            $count = IntakeCartDetail::where('idintake_cart', $detail->idintake_cart)
                ->active()
                ->count();
            if ($count == 0) IntakeCart::where('id', $detail->idintake_cart)->update(['status' => 2]);

            $intakeCart = IntakeCart::activeForId($detail->idintake_cart)
                ->first();

            if ($intakeCart) {
                $intakeCart = $intakeCart->load(
                    'supplier',
                    'details.product',
                    'company'
                );
                $intakeCart = IntakeCartResource::make($intakeCart);
            }

            DB::commit();
            return $this->successResponse('Producto eliminado con éxito.', $intakeCart);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la eliminación de un producto.', $th->getMessage());
        }
    }
}
