<?php

namespace App\Services;

use App\Http\Resources\PaymentVoucherResource;
use App\Models\AccountReceivable;
use App\Models\Cartola;
use App\Models\PaymentVoucher;
use App\Traits\HasResponse;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\Facades\JWTAuth;

/**
 * Class PaymentVoucherService
 * @package App\Services
 */
class PaymentVoucherService
{
    use HasResponse;

    public function list($params, $withPagination)
    {
        try {
            $data = PaymentVoucher::filters();

            $data = !empty($withPagination)
                ? $data->paginate($withPagination['perPage'], page: $withPagination['page'])
                : $data->get();

            $paginationTotal = null;
            if (!empty($withPagination)) $paginationTotal = $data->total();

            $data = PaymentVoucherResource::collection($data->load(
                'supplier',
                'wayToPay'
            ));

            return $this->successPaginationResponse('Lectura exitosa.', $paginationTotal, $data);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function cancel($id)
    {
        DB::beginTransaction();
        try {
            # Validar existencia del comprobante de pago
            $paymentVoucher = PaymentVoucher::activeForId($id)
                ->first()
                ->load('accountReceivableDets');

            if (!$paymentVoucher) {
                return $this->errorResponse('Comprobante de pago no válido', 400);
            }

            # Recorrer los detalles del pago y restaurar montos
            foreach ($paymentVoucher->accountReceivableDets as $accountReceivableDet) {
                $account = AccountReceivable::where('id', $accountReceivableDet->idaccount_receivable)
                    ->first()
                    ->load('ticket');

                if (!$account) continue;

                # Restaurar la deuda y los pagos en la cuenta por cobrar
                $account->update([
                    'debt'         => $account->debt + $accountReceivableDet->amount,
                    'amount_paid'  => $account->amount_paid - $accountReceivableDet->amount,
                    'paid'         => 0,
                    'payment_date' => null
                ]);

                # Eliminar el detalle del pago en la cuenta por cobrar
                $accountReceivableDet->update(['status' => 3]);
            }

            # Marcar el comprobante de pago como anulado
            $paymentVoucher->update([
                'remaining_amount' => 0,
                'status'           => 3
            ]);

            # Último registro de cartola del cliente
            $lastDetail = Cartola::where('idsupplier', $account->ticket->idcustomer)
                ->company()
                ->active()
                ->orderBy('id', 'desc')
                ->first();

            $lastBalance = $lastDetail->balance ?? 0;

            # Registrar anulación en Cartola
            Cartola::create([
                'correlative'   => Cartola::where('idcompany', $paymentVoucher->idcompany)->count() + 1,
                'idsupplier'    => $account->ticket->idcustomer,
                'idticket'      => $account->idticket,
                'idpayment_voucher' => $paymentVoucher->id,
                'type'          => 4,
                'charge'        => $paymentVoucher->amount,
                'payment'       => 0,
                'balance'       => $lastBalance + $paymentVoucher->amount,
                'gloss'         => 'Anulación del comprobante de pago ' . $paymentVoucher->correlative,
                'idcompany'     => $paymentVoucher->idcompany,
                'iduser'        => JWTAuth::user()->id,
            ]);

            DB::commit();
            return $this->successResponse('Anulación exitosa.');
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la anulación del pago.', $th->getMessage());
        }
    }
}
