<?php

namespace App\Services;

use App\Http\Resources\SalesBookResource;
use App\Http\Resources\TicketResource;
use App\Http\Resources\RouteResource;
use App\Models\Cartola;
use App\Models\Company;
use App\Models\DetailShoppingCart;
use App\Models\DetailTicket;
use App\Models\Product;
use App\Models\ShoppingCart;
use App\Models\Ticket;
use App\Models\Supplier;
use App\Models\Route;
use App\Models\DispatchGuide;
use App\Models\DispatchGuideDetail;
use App\Models\StockProduct;
use App\Models\Store;
use App\Models\RelationTickets;
use App\Models\LogApiERP;
use App\Models\WayToPay;
use App\Models\ClientAddress;
use App\Models\SettingBilling;
use App\Models\UserERP;
use App\Traits\HasResponse;
use Carbon\Carbon;
use Illuminate\Http\Exceptions\HttpResponseException;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Barryvdh\DomPDF\Facade\Pdf;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request;
use Illuminate\Support\Facades\Auth;
use Tymon\JWTAuth\Facades\JWTAuth;

class TicketService
{
    use HasResponse;

    public $igv;

    /** @var SoftnetService */
    private $softnetService;

    /** @var IntakeCartService */
    private $intakeCartService;

    /** @var ProductEntryService */
    private $productEntryService;

    public function __construct(SoftnetService $softnetService, ProductEntryService $productEntryService, IntakeCartService $intakeCartService)
    {
        $this->softnetService = $softnetService;
        $this->productEntryService = $productEntryService;
        $this->intakeCartService = $intakeCartService;
    }

    public function list($withPagination)
    {
        $tickets = Ticket::ticketFilters()->orderBy('id', 'desc');

        $tickets = !empty($withPagination)
            ? $tickets->paginate($withPagination['perPage'], page: $withPagination['page'])
            : $tickets->get();

        $tickets = TicketResource::collection($tickets->load(
            'user',
            'customer.district',
            'customer.province',
            'customer.country',
            'company',
            'detail.product.product',
            'detail.productOrigin.unitMeasure',
            'pay',
            'typeDocument',
            'saleChannel',
            'wayToPay',
            'clientAddress.district',
            'clientAddress.province',
            'clientAddress.department'
        ));

        return $this->successResponse('Lectura exitosa.', $tickets);
    }

    public function listRoute($withPagination)
    {
        $routesQuery = Route::routeFilters()->with([
            'dispatchGuides.details.ticket',
            'dispatchGuides.product',
            'dispatchGuides.warehouse',
            'user',
            'company',
        ])->company()->orderBy('id', 'desc');

        $routes = !empty($withPagination)
            ? $routesQuery->paginate($withPagination['perPage'], ['*'], 'page', $withPagination['page'])
            : $routesQuery->get();

        return $this->successResponse('Lectura exitosa.', RouteResource::collection($routes));
    }

    public function listRouteInvoice($routeId, $withPagination)
    {
        $route = Route::activeForID($routeId)->first();
        if (!$route) return $this->errorResponse('Ruta no encontrada.', 400);

        $dispatchGuideIds = DispatchGuide::where('idroute', $route->id)->pluck('id');
        $ticketIds = DispatchGuideDetail::whereIn('iddispatch_guide', $dispatchGuideIds)->pluck('idticket');
        $ticketsQuery = Ticket::ticketFilters()->whereIn('id', $ticketIds);

        $tickets = !empty($withPagination)
            ? $ticketsQuery->paginate($withPagination['perPage'], ['*'], 'page', $withPagination['page'])
            : $ticketsQuery->get();

        $tickets = TicketResource::collection($tickets->load(
            'user',
            'customer.district',
            'customer.province',
            'customer.country',
            'company',
            'detail.product.product',
            'detail.productOrigin.unitMeasure',
            'pay',
            'typeDocument',
            'saleChannel',
            'wayToPay',
            'clientAddress.district',
            'clientAddress.province',
            'clientAddress.department'
        ));

        return $this->successResponse('Lectura exitosa.', $tickets);
    }

    public function generateRoutePDF($idroute)
    {
        $route = Route::with(['dispatchGuides.details.ticket', 'dispatchGuides.product'])
            ->where('id', $idroute)
            ->first();

        if (!$route) {
            return $this->errorResponse('Ruta no encontrada.', 400);
        }

        $pdf = PDF::loadView('pdfs.ticket_route', ['route' => $route]);

        $pdf->setPaper([0, 0, 250, 800]);

        $filePath = "public/pdf/ruta_ticket_$idroute.pdf";

        Storage::put($filePath, $pdf->output());
        return response()->json(['url' => 'http://localhost:8000' . Storage::url($filePath)], 200);
    }

    public function generateDispatchPDF($idroute, $id)
    {
        $route = Route::with(['dispatchGuides.details.ticket', 'dispatchGuides.product', 'dispatchGuides.warehouse'])
            ->where('id', $idroute)
            ->first();

        if (!$route) {
            return $this->errorResponse('Ruta no encontrada.', 400);
        }

        $route->dispatchGuides = $route->dispatchGuides->filter(function ($dispatchGuide) use ($id) {
            return $dispatchGuide->idwarehouse == $id;
        })->values();


        $pdf = PDF::loadView('pdfs.ticket_dispatch', ['route' => $route]);

        $pdf->setPaper([0, 0, 250, 800]);

        $filePath = "public/pdf/dispatch_ticket_$idroute.pdf";

        Storage::put($filePath, $pdf->output());
        return response()->json(['url' => 'http://localhost:8000' . Storage::url($filePath)], 200);
    }

    public function salesBook($withPagination)
    {
        $tickets = Ticket::whereNotNull('folio')
            // ->whereIn('idtype_document', [1, 6])
            ->ticketFilters()->orderBy('id', 'desc');

        $tickets = !empty($withPagination)
            ? $tickets->paginate($withPagination['perPage'], page: $withPagination['page'])
            : $tickets->get();

        $tickets = SalesBookResource::collection($tickets->load('user', 'customer', 'company', 'typeDocument'));

        return $this->successResponse('Lectura exitosa.', $tickets);
    }

    public function register($params)
    {
        DB::beginTransaction();
        try {
            #Validar: Duplicidad de ticket
            // $ticket = Ticket::where('idshopping_cart', $params['idshopping_cart'])->active()->first();
            // if ($ticket) return $this->errorResponse('Ya se ha creado un ticket para el carrito asignado.', 400);

            #Validar que el carrito de compras tenga un cliente seleccionado
            $shoppingCart = ShoppingCart::activeForID($params['idshopping_cart'])->company()->first();
            if (!$shoppingCart) return $this->errorResponse('El carrito de compras no es válido.', 400);

            #Validar que el carrito de compras tenga productos añadidos
            $detShoppingCart = DetailShoppingCart::where('idshopping_cart', $shoppingCart->id)->active()->get();
            if (count($detShoppingCart) == 0) return $this->errorResponse('El carrito de compras no tiene productos añadidos.', 400);

            $customer = Supplier::where('id', $shoppingCart->idcustomer)->active()->first();
            if (!$customer) return $this->errorResponse('El cliente no es valido.', 400);

            #Generar numero de ticket
            $nro_ticket = Ticket::orderBy('id', 'desc')->get();
            $nro_ticket = count($nro_ticket) > 0 ? count($nro_ticket) + 1 : 1;

            $discountGlobalMount = $shoppingCart->discounttype === 2 ? ($shoppingCart->sub_total * ($shoppingCart->global_discount / 100)) :  $shoppingCart->global_discount;

            $ticket = Ticket::create([
                'date'              => Carbon::now()->toDateString(),
                'nro_ticket'        => $nro_ticket,
                'idshopping_cart'   => $shoppingCart->id,
                'iduser_created'    => $shoppingCart->iduser_created,
                'idcustomer'        => $customer->id,
                'idclient_address'  => $params['idclient_address'] ?? null,
                'seller_rut'        => $params['seller_rut'] ?? null,
                'gross_total'       => $shoppingCart->gross_total,
                'net_total'         => $shoppingCart->net_total,
                'IVA'               => $shoppingCart->IGV,
                'idcompany'         => $shoppingCart->idcompany,
                'idtype_document'   => $params['idtype_document'],
                'idsale_channel'    => $params['idsale_channel'] ??  null,
                // 'payment_condition' => $params['payment_condition'],
                'idway_to_pay'      => $params['idway_to_pay'] ?? null,
                'bussines_area'     => $params['bussines_area'] ?? null,
                'observation'       => $params['observation'] ?? null,
                'folio'             => $params['folio'] ?? null,
                'total_discount'    => $discountGlobalMount,
                'sub_total'         => $shoppingCart->sub_total,
                'total'             => $shoppingCart->total,
                'status'            => $params['status'] ?? '1',
                'idsalesman'        => $params['idsalesman'] ?? null
            ]);

            #Crear Detalle
            $details = [];
            foreach ($detShoppingCart as $detail) {
                $details[] = [
                    'idticket' => $ticket->id,
                    'idpricelist_product' => $detail->idpricelist_product,
                    'idproduct' => $detail->idproduct,
                    'amount' => $detail->quantity,
                    'price' => $detail->price,
                    'discount' => $detail->discount,
                    'discounttype' => $detail->discounttype,
                    'gross_subtotal' => $detail->gross_subtotal,
                    'net_subtotal' => $detail->net_subtotal,
                ];
            }

            if (!empty($details)) DetailTicket::insert($details);

            DB::commit();
            return $this->successResponse('Ticket creado con éxito.', $ticket);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la creación de un ticket.', $th->getMessage());
        }
    }

    public function registerRelation($params)
    {
        DB::beginTransaction();
        try {
            $idtickets_child = $params['idtickets_child'];

            if (empty($idtickets_child)) {
                return $this->errorResponse('No se han proporcionado tickets hijos.', 400);
            }

            # Obtener todos los tickets hijos
            $tickets_child = Ticket::whereIn('id', $idtickets_child)->get();

            if ($tickets_child->isEmpty()) {
                return $this->errorResponse('No se encontraron tickets hijos válidos.', 400);
            }

            # Validar que todos los tickets hijos tengan los mismos datos comerciales
            $keysToCheck = [
                'idcustomer',
                'idclient_address',
                'idcompany',
                'idway_to_pay',
                'bussines_area',
            ];

            foreach ($keysToCheck as $key) {
                if ($tickets_child->groupBy($key)->count() > 1) {
                    return $this->errorResponse("Los tickets hijos tienen valores diferentes en '$key'. No pueden agruparse.", 400);
                }
            }

            # Obtener referencias
            $referencias = [];
            foreach ($tickets_child as $ticket_child) {
                $referencias[] = [
                    'Tipo'      => '52',
                    'Folio'     => $tickets_child->folio,
                    'Fecha'     => $tickets_child->date,
                    'Razon'     => 'facturador'
                ];
            }

            # Obtener el número del último ticket y asignar el siguiente número
            $lastTicket = Ticket::orderBy('id', 'desc')->first();
            $nro_ticket = $lastTicket ? $lastTicket->id + 1 : 1;

            # Datos obtenidos del primer ticket hijo (ya validado)
            $ticketChild = $tickets_child->first();

            # Calcular sumas manualmente
            $gross_total = $tickets_child->sum('gross_total');
            $net_total   = $tickets_child->sum('net_total');
            $IVA         = $tickets_child->sum('IVA');
            $sub_total   = $tickets_child->sum('sub_total');
            $total       = $tickets_child->sum('total');

            # Crear el ticket padre
            $ticket_parent = Ticket::create([
                'date'              => Carbon::now()->toDateString(),
                'nro_ticket'        => $nro_ticket,
                'type'              => 2, // Grupal
                'idshopping_cart'   => null,
                'iduser_created'    => $ticketChild->iduser_created,
                'idcustomer'        => $ticketChild->idcustomer,
                'idclient_address'  => $ticketChild->idclient_address ?? null,
                'seller_rut'        => $ticketChild->seller_rut ?? null,
                'gross_total'       => $gross_total,
                'net_total'         => $net_total,
                'IVA'               => $IVA,
                'idcompany'         => $ticketChild->idcompany,
                'idtype_document'   => 2, // Factura
                'idsale_channel'    => $ticketChild->idsale_channel ?? null,
                'idway_to_pay'      => $ticketChild->idway_to_pay ?? null,
                'bussines_area'     => $ticketChild->bussines_area ?? null,
                'observation'       => $ticketChild->observation ?? null,
                'folio'             => null,
                'total_discount'    => 0,
                'sub_total'         => $sub_total,
                'total'             => $total,
                'status'            => '1'
            ]);

            # Crear relaciones en una sola inserción
            $relations = [];
            foreach ($idtickets_child as $idticket_child) {
                $relations[] = [
                    'idticket_parent' => $ticket_parent->id,
                    'idticket_child'  => $idticket_child
                ];
            }

            # Inserción masiva
            RelationTickets::insert($relations);

            # Actualizar los tickets hijos: cambiar `type` a 3
            Ticket::whereIn('id', $idtickets_child)->update(['type' => 3]);

            # Factura
            $receiver = $ticket_parent['customer']['rut'];
            $netAmount = $ticket_parent->sub_total;
            $exemptAmount = $ticket_parent->mnt_excent ?? 0;
            $discountHeader = $ticket_parent->total_discount;
            $folio = 0;
            $businessArea = $ticket_parent->bussines_area;
            $seller = $ticket_parent->seller_rut ?? '15134573-5'; # TEST - DE LA SESIÓN
            $observation = $ticket_parent->observation ?? 'obs';
            $clientAddress = ClientAddress::with('district')->where('id', $ticket_parent->idclient_address)->first();
            $address = $clientAddress ? $clientAddress->id : "SANTIAGO";
            $two = $ticket_parent->seller_rut ?? '15134573-5'; # TEST - DE LA SESIÓN
            #Forma de Pago
            $wayPay = WayToPay::activeForID($ticket_parent->idway_to_pay)->first();
            $condicionPago = $wayPay->iderp;
            #bodega 
            $store = Store::where('idcompany', $ticket_parent->idcompany)->where('status', 1)->first();
            if (!$store) return $this->errorResponse('La bodega no valida', 400);
            $detailJson = [];
            # Obtener detalles de los tickets hijos y agrupar por producto
            $detailTickets = DetailTicket::whereIn('idticket', $idtickets_child)
                ->active()
                ->get()
                ->groupBy('idproduct'); # Agrupar por producto

            $detailJson = [];

            foreach ($detailTickets as $idproduct => $details) {
                $totalAmount = $details->sum('amount'); # Sumar cantidades del mismo producto
                $totalDiscount = $details->sum(function ($detail) {
                    return $detail->discounttype == 1
                        ? $detail->discount
                        : ($detail->amount * $detail->price) * ($detail->discount / 100);
                });

                # Tomar el primer registro como base para extraer los datos del producto
                $firstDetail = $details->first();

                $detailJson[] = [
                    "Codigo"        => $firstDetail['productOrigin']['code_erp'], # Código ERP
                    "Cantidad"      => $totalAmount, # Nueva cantidad sumada
                    "Descuento"     => $totalDiscount, # Nueva suma de descuentos
                    "Precio"        => $firstDetail->price,
                    "TipoDescuento" => "1",
                    "Afecto"        => true,
                    "Bodega"        => $store->iderp,
                    "Nombre"        => $firstDetail['productOrigin']['name']
                ];
            }
            # En caso el cliente no exista en erp
            $dataNewClient = $ticket_parent['customer'];
            if ($clientAddress->district) {
                $districtName = $clientAddress->district->name;
            } else {
                $districtName = 'SANTIAGO';
            }

            $newClient = (object) [
                'NombreRZ' => $dataNewClient['name_rz'],
                'NombreFantasia' => $dataNewClient['fantasy_name'],
                'Rut' => $dataNewClient['rut'],
                'Giro' => $dataNewClient['giro'],
                'Direccion' => $clientAddress->address,
                'Telefono' => $dataNewClient['phone'],
                'Fax' => $dataNewClient['fax'],
                'Comuna' => $districtName,
                'Ciudad' => $districtName,
                'Pais' => 'Chile',
                'Email' => $dataNewClient['email'],
                'Web' => $dataNewClient['web'],
                'NombreCTO' => $dataNewClient['name_rz'],
                'EmailCTO' => $dataNewClient['email'],
                'TelefonoCTO' => $dataNewClient['phone'],
                'CelularCTO' => $dataNewClient['phone']
            ];

            $body = '[
                {
                    "Encabezado": {
                        "Receptor": "' . $receiver . '",
                        "MontoNeto": ' . $netAmount . ',
                        "MontoExento": "' . $exemptAmount . '",
                        "Descuento": "' . $discountHeader . '",
                        "TipoDocumento": "33",
                        "Folio": ' . $folio . ',
                        "AreaNegocio": "' . $businessArea . '",
                        "Vendedor": "' . $seller . '",
                        "Observacion": "' . $observation . '",
                        "Direccion": "' . $address . '"
                    },
                    "NuevoCliente": ' . json_encode($newClient) . ',
                    "Detalle": ' . json_encode($detailJson) . ',
                    "Adicional": {
                        "Uno": ' . $condicionPago . ',
                        "Dos": "' . $two . '"
                    }
                }
            ]';
            $userErp = UserERP::where('iduser', Auth::user()->id)->first();
            if (!$userErp) return $this->errorResponse('El usuario no cuenta con integracion ERP.', 400);

            $token = $this->softnetService->getTokenSoftnet($userErp);
            # Inicio de petición
            $client = new Client();
            $headers = ['token' => $token];

            $request = new Request('POST', 'api2.softnet.cl/facturaFacturador', $headers, $body);
            $endpoint = 'api2.softnet.cl/facturaFacturador';

            # Esperar petición
            $res = $client->sendAsync($request)->wait();
            $res = $res->getBody();
            $resJson = json_decode($res)[0] ?? false;

            # Actualizar ticket
            $ticket_parent->update([
                'status_invoice'    => 1,
                'url_invoice'       => $resJson->pdf ?? null,
                'folio'             => $resJson->folio ?? null
            ]);

            # Generar registro log del uso de API
            LogApiERP::create([
                'idticket'  => $ticket_parent->id,
                'endpoint'  => $endpoint,
                'metodo'    => 'POST',
                'entrada'   => $body,
                'respuesta' => $res ?? ''
            ]);

            DB::commit();
            return $this->successResponse('Relación generada con éxito.', $ticket_parent);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('Error durante la relación de tickets.', $th->getMessage());
        }
    }

    public function delete($id)
    {
        DB::beginTransaction();
        try {
            #Validar carrito de compras
            $ticket = Ticket::activeForID($id)->company()->first();
            if (!$ticket) return $this->errorResponse('Ticket no válido.', 400);

            $ticket->update(['status' => 2]);

            #Eliminar detalles
            DetailTicket::where('idticket', $ticket->id)->active()->update(['status' => 2]);

            #si es una factura o boleta emitida, generamos una nota de credito
            if (in_array($ticket->idtype_document, [1, 2]) && in_array($ticket->status_invoice, [1, 2])) {

                #Generar numero de ticket
                $nro_ticket = Ticket::orderBy('id', 'desc')->get();
                $nro_ticket = count($nro_ticket) > 0 ? count($nro_ticket) + 1 : 1;

                $discountGlobalMount = 0;

                $ticketNC = Ticket::create([
                    'date'              => Carbon::now()->toDateString(),
                    'nro_ticket'        => $nro_ticket,
                    'idshopping_cart'   => $ticket->idshopping_cart,
                    'iduser_created'    => $ticket->iduser_created,
                    'idcustomer'        => $ticket->idcustomer,
                    'idclient_address'  => $ticket->idclient_address ?? null,
                    'seller_rut'        => $ticket->seller_rut ?? null,
                    'gross_total'       => $ticket->gross_total,
                    'net_total'         => $ticket->net_total,
                    'IVA'               => $ticket->IVA,
                    'idcompany'         => $ticket->idcompany,
                    'idtype_document'   => 3,
                    'idsale_channel'    => $ticket->idsale_channel ??  null,
                    'idway_to_pay'      => $ticket->idway_to_pay ?? null,
                    'bussines_area'     => $ticket->bussines_area ?? null,
                    'observation'       => $ticket->observation ?? null,
                    'total_discount'    => $discountGlobalMount,
                    'sub_total'         => $ticket->sub_total,
                    'total'             => $ticket->total,
                    'status'            => '1'
                ]);

                # Variables para Encabezado
                $receiver = $ticketNC['customer']['rut'];
                $netAmount = $ticketNC->sub_total;
                $exemptAmount = $ticketNC->mnt_excent ?? 0;
                $discountHeader = $ticketNC->total_discount;
                $discountType = "1";
                $folio = $ticketNC->folio; # TEST
                $businessArea = $ticketNC->bussines_area;
                $seller = $ticketNC->seller_rut ?? '15134573-5'; # TEST - DE LA SESIÓN
                $observation = $ticketNC->observation ?? 'obs';
                #Forma de Pago
                $wayPay = WayToPay::activeForID($ticketNC->idway_to_pay)->first();
                $condicionPago = $wayPay->iderp;
                $one = $wayPay->name;
                $two = $ticketNC->seller_rut ?? '15134573-5';
                #bodega 
                $store = Store::where('idcompany', $ticket->idcompany)->where('status', 1)->first();
                if (!$store) return $this->errorResponse('La bodega no valida', 400);
                # Variables para Detalle
                $detailJson = [];

                if ($ticket->type == 1) {
                    $detailTicket = DetailTicket::where('idticket', $ticket->id)->get();
                    foreach ($detailTicket as $detail) {

                        $detailJson[] = [
                            "Codigo"    => $detail['productOrigin']['code_erp'], # code_erp o code_system
                            "Cantidad"  => $detail->amount,
                            "Descuento" => $detail->discounttype == 1 ? $detail->discount : ($detail->amount * $detail->price)  * ($detail->discount / 100),
                            "Precio"    => $detail->price,
                            "TipoDescuento" =>  "1",
                            "Afecto"    => true,
                            "Bodega"    => $store->iderp, # TEST - Se obtiene la bodega cuando se importa, si es creado de 0 no tiene warehouse, mandar uno por defecto?
                            "Nombre"    => $detail['productOrigin']['name']
                        ];
                    }
                }

                if ($ticket->type == 2) {
                    $idtickets_child = RelationTickets::where('idticket_parent', $ticket->id)
                        ->pluck('idticket_child')
                        ->toArray();
                    $detailTickets = DetailTicket::whereIn('idticket', $idtickets_child)
                        ->active()
                        ->get()
                        ->groupBy('idproduct');

                    foreach ($detailTickets as $idproduct => $details) {
                        $totalAmount = $details->sum('amount'); # Sumar cantidades del mismo producto
                        $totalDiscount = $details->sum(function ($detail) {
                            return $detail->discounttype == 1
                                ? $detail->discount
                                : ($detail->amount * $detail->price) * ($detail->discount / 100);
                        });

                        # Tomar el primer registro como base para extraer los datos del producto
                        $firstDetail = $details->first();

                        $detailJson[] = [
                            "Codigo"        => $firstDetail['productOrigin']['code_erp'], # Código ERP
                            "Cantidad"      => $totalAmount, # Nueva cantidad sumada
                            "Descuento"     => $totalDiscount, # Nueva suma de descuentos
                            "Precio"        => $firstDetail->price,
                            "TipoDescuento" => "1",
                            "Afecto"        => true,
                            "Bodega"        => $store->iderp,
                            "Nombre"        => $firstDetail['productOrigin']['name']
                        ];
                    }
                }

                $folio = 0;

                $body = '[
                    {
                      "Encabezado": {
                        "Receptor": "' . $receiver . '",
                        "Folio": "' . $folio . '",
                        "MontoNeto": ' . $netAmount . ',
                        "MontoExento": "' . $exemptAmount . '",
                        "Descuento": "' . $discountHeader . '",
                        "TipoDocumento": 61,
                        "AreaNegocio": "' . $businessArea . '",
                        "Vendedor": "' . $seller . '",
                        "CondicionPago": "' . $condicionPago . '",
                        "Observacion": "' . $observation . '"
                      },
                      "Detalle": ' . json_encode($detailJson) . ',
                      "Referencia": {
                        "Numero":"' . 1 . '",
                        "Tipo":"' . 33 . '",
                        "Folio":"' . $ticket->folio . '",
                        "Fecha":"' . $ticket->date . '",
                        "CodigoNC":"' . 1 . '",
                        "Razon":"' . 'Anulacion' . '"
                      },
                      "Adicional": {
                        "Uno": "' . $one . '",
                        "Dos": "' . $two . '"
                      }
                    }
                  ]';
                //dd($body);
                $userErp = UserERP::where('iduser', Auth::user()->id)->first();
                if (!$userErp) return $this->errorResponse('El usuario no cuenta con integracion ERP.', 400);

                $token = $this->softnetService->getTokenSoftnet($userErp);

                # Inicio de petición
                $client = new Client();
                $headers = ['token' => $token];
                //dd($token, $body, 'api.softnet.cl/notaCredito');
                $request = new Request('POST', 'api2.softnet.cl/notaCreditoFacturador', $headers, $body);
                $endpoint = 'api2.softnet.cl/notaCreditoFacturador';

                # Esperar petición
                $res = $client->sendAsync($request)->wait();
                $res = $res->getBody();
                $resJson = json_decode($res)[0] ?? false;

                if (!$resJson) return $this->errorResponse('Inténtelo nuevamente en unos momentos.', 400);

                # Actualizar ticket
                $ticketNC->update([
                    'status_invoice'    => 1,
                    'url_invoice'       => $resJson->pdf ?? null,
                    'folio'             => $folio ?? null
                ]);

                # Generar registro log del uso de API
                LogApiERP::create([
                    'idticket'  => $id,
                    'endpoint'  => $endpoint,
                    'metodo'    => 'POST',
                    'entrada'   => $body,
                    'respuesta' => $res ?? ''
                ]);
            }

            if (in_array($ticket->status_invoice, [1, 2])) {

                $intakeCartId = 0;

                if ($ticket->type == 1) {

                    $detailTicket = DetailTicket::where('idticket', $ticket->id)->get();
                    foreach ($detailTicket as $detail) {
                        $payloadIntakeCart = [
                            'idsupplier'    => $ticket->idcustomer,
                            'idcompany'     => $ticket->idcompany,
                            'idproduct'     => $detail->idproduct,
                            'quantity'      => $detail->amount
                        ];

                        $response = $this->intakeCartService->register($payloadIntakeCart);
                        if (!$response->original['status']) return $response;

                        $intakeCart = $response->original['data']['detail'];
                        $intakeCartId = $intakeCart->id;
                    }
                } else {
                    $idtickets_child = RelationTickets::where('idticket_parent', $ticket->id)
                        ->pluck('idticket_child')
                        ->toArray();
                    $detailTickets = DetailTicket::whereIn('idticket', $idtickets_child)
                        ->active()
                        ->get()
                        ->groupBy('idproduct');
                    foreach ($detailTickets as $idproduct => $details) {
                        $totalAmount = $details->sum('amount'); # Sumar cantidades del mismo producto

                        # Tomar el primer registro como base para extraer los datos del producto
                        $firstDetail = $details->first();

                        $payloadIntakeCart = [
                            'idsupplier'    => $ticket->idcustomer,
                            'idcompany'     => $ticket->idcompany,
                            'idproduct'     => $firstDetail->idproduct,
                            'quantity'      => $totalAmount
                        ];
                        $response = $this->intakeCartService->register($payloadIntakeCart);
                        if (!$response->original['status']) return $response;

                        $intakeCart = $response->original['data']['detail'];
                        $intakeCartId = $intakeCart->id;
                    }
                }

                $payloadProductEntry = [
                    'idintake_cart' => $intakeCartId
                ];

                $validate = $this->productEntryService->register($payloadProductEntry);
                if (!$validate->original['status']) return $validate;

                #Insertamos en cartola como abono

                $lastDetail = Cartola::where('idsupplier', $ticketNC->idcustomer)
                    ->company()
                    ->active()
                    ->orderBy('id', 'desc')
                    ->first();

                $lastBalance = $lastDetail->balance ?? 0;

                Cartola::create([
                    'correlative' => Cartola::where('idcompany', $ticketNC->idcompany)->count() + 1,
                    'idsupplier' => $ticketNC->idcustomer,
                    'idticket' => $ticketNC->id,
                    'type' => 5,
                    'charge' => $ticketNC->total,
                    'payment' => 0,
                    'balance' => $lastBalance - $ticketNC->total,
                    'gloss' => 'Anulacion de factura - ' . $ticketNC->folio,
                    'idcompany' => $ticketNC->idcompany,
                    'iduser' => JWTAuth::user()->id,
                ]);
            }

            DB::commit();
            return $this->successResponse('Ticket eliminado con éxito.', $ticket);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la eliminado de un ticket.', $th->getMessage());
        }
    }

    public function generateDocument($params)
    {
        try {
            #Validar parámetros
            $validator = Validator::make($params, [
                'idticket'  => 'required|numeric|validate_ids_exist:Ticket',
                'idcompany' => 'required|numeric|validate_ids_exist:Company',
                'folio'     => 'nullable|numeric|min:1',

                'references' => 'nullable|array|min:1',
                'references.*.TpoDocRef' => 'required|numeric',
                'references.*.FolioRef'  => 'required|numeric|min:0',
                'references.*.FchRef'    => 'required|date',
                'references.*.CodRef'    => 'required|numeric|in:1,2,3',
                'references.*.RazonRef'  => 'required|min:5|max:80'
            ]);
            if ($validator->fails()) {
                $errors = $validator->errors();
                $primerArray = reset($errors);
                $primerError = reset($primerArray);
                $message = $primerError[0];

                throw new HttpResponseException($this->errorResponse($message, 400));
            }

            #Validar: Ticket con documento emitido
            $ticket = Ticket::activeForId($params['idticket'])->where('status_invoice', 1)->first();
            if ($ticket) return $this->errorResponse('Ya se ha emitido el comprobante del ticket seleccionado.', 400);

            $company = Company::activeForID($params['idcompany'])->get()->load('district');
            $ticket  = Ticket::activeForId($params['idticket'])->company()->get()->load('customer.district', 'detail');

            $customer           = $ticket[0]['customer'];
            $detailsTicket      = $ticket[0]['detail'];
            $districtCustomer   = $ticket[0]['customer']['district'];
            $ticket             = $ticket[0];
            $disctrictCompany   = $company[0]['district'];
            $company            = $company[0];

            #Obtener el token del usuario erp de la empresa
            // $userErp = UserERP::where('id', $company->rut)->first(); #descomentar
            $userErp = (object)[
                'rut'                => '22222222-2',
                'user'               => 'usuario',
                'password'           => 'demoerp',
                'encrypted_password' => Crypt::encryptString('demoerp'),
                'status'             => 1
            ];
            if (!$userErp) return $this->errorResponse('La empresa no registra su usuario del ERP para comenzar a facturar.', 400);
            $token    = $this->softnetService->getTokenSoftnet($userErp);
            $tokenErp =  Crypt::encryptString($token);

            $param['token_erp'] = $tokenErp;

            $TipoDTE = 0;
            switch ($ticket->idtype_document) {
                #boleta
                case 1:
                    $TipoDTE = 39;
                    break;

                #factura
                case 2:
                    $TipoDTE = 33;
                    break;

                #nota de credito
                case 3:
                    $TipoDTE = 61;
                    break;

                #nota de debito
                case 4:
                    $TipoDTE = 56;
                    break;

                #cotizacion
                case 5:
                    $TipoDTE = 777;
                    break;

                default:
                    break;
            }

            $Encabezado = (object)[
                'IdDoc' => (object)[
                    'TipoDTE' => $TipoDTE,
                    'Folio'   => $params['folio'] ?? 1,
                    'FchEmis' => Carbon::now()->toDateString(),
                ],
                'Emisor' => (object)[
                    'RUTEmisor'     => $company->rut,
                    'RznSocEmisor'  => $company->business_name,
                    'GiroEmis'      => $company->giro,
                    'Telefono'      => $company->phone,
                    'Acteco'        => 722000, #Por consultar y modificar
                    'DirOrigen'     => $company->address,
                    'CmnaOrigen'    => $disctrictCompany->name,
                    'CiudadOrigen'  => $disctrictCompany->name
                ],
                'Receptor' => (object)[
                    'RUTRecep'      => $customer->rut,
                    'RznSocRecep'   => $customer->name_rz,
                    'GiroRecep'     => $customer->giro,
                    'DirRecep'      => $customer->address,
                    'CmnaRecep'     => $districtCustomer->name,
                    'CiudadRecep'   => $districtCustomer->name
                ],
                'Totales' => (object)[
                    'MntNeto'   => round($ticket->net_total),
                    'IVA'       => round($ticket->IVA),
                    'MntTotal'  => round($ticket->gross_total)
                ]
            ];

            #Si es factura agregar monto excento a totales
            if ($ticket->idtype_document == 2) {
                $Encabezado->Totales->MntExe = $ticket->mnt_excent;
            }

            $detalles = [];
            foreach ($detailsTicket as $row) {
                $NmbItem = Product::whereHas('detailPriceListProduct', fn($q) => $q->where('id', $row->idpricelist_product))->value('name');
                $detalles[] = (object)[
                    'NmbItem'   => $NmbItem,
                    'PrcItem'   => strval($row->price),
                    'QtyItem'   => strval($row->amount),
                    'MontoItem' => strval($row->price * $row->amount)
                ];
            }

            $Documento = (object)[
                'Encabezado' => $Encabezado,
                'Detalle'    => $detalles
            ];

            #Si es nota de credito o debito agregar referencias
            if ($ticket->idtype_document == 3 || $ticket->idtype_document == 4) {
                $referencias = [];
                foreach ($params['references'] as $row) {
                    $referencias[] = (object)[
                        'TpoDocRef' => $row->TpoDocRef,
                        'FolioRef'  => $row->FolioRef,
                        'FchRef'    => $row->FchRef,
                        'CodRef'    => $row->CodRef,
                        'RazonRef'  => $row->RazonRef
                    ];
                }
                $Documento->Referencia = $referencias;
            }

            $DTE = [
                'DTE' => [
                    'Documento'     => $Documento,
                    'Adicionales'   => (object)[]
                ]
            ];

            $facturaERP = $this->softnetService->EmiteFactura($param, $DTE, $ticket->id, $ticket->idtype_document);
            if (!isset($facturaERP['mensaje']) || $facturaERP['mensaje'] != "Dte generado con exito.") return $this->errorResponse("Error al crear el documento.", 400);

            $urlpdfDTE = $this->softnetService->consultaDTE($param, $facturaERP['tipo'], $facturaERP['folio'], $company->rut);

            $ticket = Ticket::find($params['idticket'])->first();

            if (!empty($urlpdfDTE)) {
                $pdfContent = file_get_contents($urlpdfDTE);

                $folder = "$company->business_name/documentos/";
                $filename = 'doc' . $facturaERP['folio'] . '_' . $customer->rut . '.pdf';
                $filePath = $folder . '/' . $filename;

                if (!Storage::disk('public')->exists($folder)) Storage::disk('public')->makeDirectory($folder);
                Storage::disk('public')->put($filePath, $pdfContent);

                $ticket->update([
                    'status_invoice' => 1,
                    'url_invoice'    => "storage/$filePath",
                    'folio'          =>  $facturaERP['folio']
                ]);
            } else {
                $ticket->update([
                    'status_invoice' => 1,
                    'folio'          =>  $facturaERP['folio']
                ]);
            }

            return $this->successResponse('Documento emitido con éxito');
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function generateRoute($params)
    {
        DB::beginTransaction();
        try {
            # Validar que se proporcionaron tickets
            $ticketIds = $params['idsticket'] ?? [];
            if (empty($ticketIds)) {
                return $this->errorResponse('No se proporcionaron tickets.', 400);
            }

            # Obtener los tickets válidos con sus detalles y direcciones
            $tickets = Ticket::whereIn('id', $ticketIds)
                ->where('status_invoice', 1)
                ->with(['detail', 'clientAddress.district'])
                ->active()
                ->get();

            if ($tickets->isEmpty()) {
                return $this->errorResponse('No se encontraron tickets válidos.', 404);
            }

            # Obtener el ultimo correlativo de ruta para el dia
            $correlative = 1;
            $lastRoute = Route::where('idcompany', auth()->user()->idcompany)->where('date', now())->first();
            if ($lastRoute) $correlative = $lastRoute->n_route + 1;

            # Crear la ruta (único registro)
            $route = Route::create([
                'date' => $params['date'],
                'n_route' => $correlative,
                'iduser_created' => auth()->id(),
                'patente' => $params['patente'] ?? null,
                'driver' => $params['driver'] ?? null,
                'observations' => $params['observations'] ?? null,
                'idcompany' => auth()->user()->idcompany,
                'status' => 1,
            ]);

            # Obtener el stock por producto y bodega
            $stockProducts = StockProduct::with('warehouse.district')
                ->get()
                ->groupBy('idproduct');

            $productQuantities = [];
            $ticketDetails = [];

            $storePrincipal = Store::where('idcompany', auth()->user()->idcompany)->where('name', 'general')->first();

            foreach ($tickets as $ticket) {

                if ($ticket->type == 1) {
                    foreach ($ticket->detail as $detail) {
                        $productId = $detail->idproduct;
                        $quantity = $detail->amount;

                        $warehouseId = null;

                        if (isset($stockProducts[$productId]) && count($stockProducts[$productId]) > 0) {
                            $warehouseId = $stockProducts[$productId][0]->idwarehouse; // Acceder al primer elemento
                        }

                        # Si no se encontró una coincidencia, tomar bodega principal
                        if (!$warehouseId) {
                            $warehouseId = $storePrincipal->id;
                        }

                        # Acumular cantidades por producto y bodega
                        if (!isset($productQuantities[$warehouseId][$productId])) {
                            $productQuantities[$warehouseId][$productId] = 0;
                        }
                        $productQuantities[$warehouseId][$productId] += $quantity;

                        # Preparar detalles de tickets
                        $ticketDetails[] = [
                            'id_ticket' => $ticket->id,
                            'id_product' => $productId,
                            'quantity' => $quantity,
                            'id_warehouse' => $warehouseId,
                        ];

                        # Ultimo stock  
                        $lastStock = StockProduct::where('idproduct', $productId)
                            ->when(
                                $warehouseId != null,
                                fn($q) => $q->where('idwarehouse', $warehouseId)
                            )->active()
                            ->orderBy('id', 'desc')
                            ->first();

                        $newStock = $lastStock ? $lastStock->stock_system - $quantity : -$quantity;

                        # Insertar las salidas de inventario
                        StockProduct::create([
                            'stock_system'      => $newStock,
                            'registration_date' => Carbon::now()->toDateTimeString(),
                            'last_update'       => $lastStock->registration_date ?? Carbon::now()->toDateTimeString(),
                            'idproduct'         => $productId,
                            'idwarehouse'       => $warehouseId,
                            'idcompany'         => auth()->user()->idcompany,
                            'type_movement'     => 2,
                            'initial_stock'     => $lastStock->stock_system ?? 0,
                            'difference'        => -$quantity,
                            'folio'             => $ticket->folio ?? 0,
                            'type_dte'          => $ticket->typeDocument->cod_erp ?? '',
                            'date_dte'          => Carbon::now()->toDateTimeString(),
                        ]);
                    }
                }
            }

            # Preparar datos para insertar las guías de despacho
            $dispatchGuides = [];
            foreach ($productQuantities as $warehouseId => $products) {
                foreach ($products as $productId => $quantity) {
                    $dispatchGuides[] = [
                        'date' => now(),
                        'n_document' => uniqid('GUIA-'),
                        'idroute' => $route->id,
                        'idproduct' => $productId,
                        'idwarehouse' => $warehouseId,
                        'quantity' => $quantity,
                        'idcompany' => auth()->user()->idcompany,
                        'status' => 1,
                        'created_at' => now(),
                        'updated_at' => now(),
                    ];
                }
            }

            # Inserción masiva de guías de despacho
            DispatchGuide::insert($dispatchGuides);

            # Recuperar los IDs generados de las guías de despacho
            $dispatchGuidesMap = DispatchGuide::where('idroute', $route->id)
                ->pluck('id', 'idproduct')
                ->toArray();

            # Preparar los detalles de las guías de despacho
            $dispatchGuideDetails = [];
            foreach ($ticketDetails as $detail) {
                $productId = $detail['id_product'];

                if (isset($dispatchGuidesMap[$productId])) {
                    $dispatchGuideDetails[] = [
                        'iddispatch_guide' => $dispatchGuidesMap[$productId],
                        'idticket' => $detail['id_ticket'],
                        'idproduct' => $productId,
                        'quantity' => $detail['quantity'],
                        'idcompany' => auth()->user()->idcompany,
                        'status' => 1,
                        'created_at' => now(),
                        'updated_at' => now(),
                    ];
                }
            }

            # Inserción masiva de detalles de guías de despacho
            DispatchGuideDetail::insert($dispatchGuideDetails);

            # Actualizar el status_invoice de los tickets a 2
            Ticket::whereIn('id', $ticketIds)->update(['status_invoice' => 2]);

            DB::commit();
            return $this->successResponse('Ruta generada con éxito.');
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->errorResponse('Error generando la ruta: ' . $th->getMessage(), 500);
        }
    }

    public function updateRoute($id, $params)
    {
        DB::beginTransaction();
        try {
            $route = Route::activeForID($id)->first();
            if (!$route) return $this->errorResponse('Ruta no encontrada.', 400);

            $dispatchGuideIds = DispatchGuide::where('idroute', $route->id)->pluck('id');
            $ticketIds = DispatchGuideDetail::whereIn('iddispatch_guide', $dispatchGuideIds)->pluck('idticket');
            $ticketsQuery = Ticket::whereIn('id', $ticketIds)->where('status_route', 2)->get();

            if (count($ticketsQuery) > 0) {
                return $this->errorResponse("No se puede cerrar la ruta. Hay un documento con un estado pospuesto.", 400);
            }

            $route->update($params);

            DB::commit();
            return $this->successResponse('Ruta cerrada con éxito.');
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->errorResponse('Error actualizando la ruta: ' . $th->getMessage(), 500);
        }
    }


    public function confirmTicket($id, $params)
    {
        DB::beginTransaction();
        try {
            #Validar parámetros
            $validator = Validator::make($params, [
                'idtype_document'  => 'required|integer|min:1',
                'idway_to_pay'     => 'nullable|integer|min:1',
                'bussines_area'    => 'nullable|integer|min:1',
                'idsalesman'       => 'nullable|integer|min:1'
            ]);
            if ($validator->fails()) {
                $errors = $validator->errors();
                $primerArray = reset($errors);
                $primerError = reset($primerArray);
                $message = $primerError[0];

                throw new HttpResponseException($this->errorResponse($message, 400));
            }

            # Validar que exista el ticket
            $ticket = Ticket::where('id', $id)->where('status', 0)->first();
            if (!$ticket) return $this->errorResponse('Ticket seleccionado no válido.', 400);

            # Validar que exista forma de pago
            if (!$ticket->idway_to_pay && !isset($params['idway_to_pay'])) {
                return $this->errorResponse('Ingrese una forma de pago para confirmar el pedido.', 400);
            }

            # Validar que exista area de negocio
            if (!$ticket->bussines_area && !isset($params['bussines_area'])) {
                return $this->errorResponse('Ingrese un área de negocio para confirmar el pedido.', 400);
            }

            $params['status'] = 1;

            $ticket->update($params);

            ShoppingCart::where('id', $ticket->idshopping_cart)->update(['status_ticket' => 1]);

            DB::commit();
            return $this->successResponse('Pedido generado con éxito.', $ticket);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la creación de un ticket.', $th->getMessage());
        }
    }

    public function changeStatus($id, $params)
    {
        DB::beginTransaction();
        try {
            # Validar que exista el ticket
            $ticket = Ticket::where('id', $id)->first();
            if (!$ticket) return $this->errorResponse('Ticket seleccionado no válido.', 400);

            $ticket->update(['status' => $params['status']]);

            DB::commit();
            return $this->successResponse('Generado con éxito.', $ticket);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la creación de un ticket.', $th->getMessage());
        }
    }

    public function changeStatusRoute($id, $params)
    {
        DB::beginTransaction();
        try {
            # Validar que exista el ticket
            $ticket = Ticket::where('id', $id)->first();
            if (!$ticket) return $this->errorResponse('Ticket seleccionado no válido.', 400);

            $ticket->update(['status_route' => $params['status_route']]);

            DB::commit();
            return $this->successResponse('Actualizado con éxito.', $ticket);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('durante la creación de un ticket.', $th->getMessage());
        }
    }

    public function partialCancelation($id, $params)
    {
        DB::beginTransaction();
        try {
            # Validar parámetros
            $validator = Validator::make($params, [
                'changes'             => ['required', 'array'],
                'changes.*.id'        => ['required_with:changes', 'integer', 'min:1'],
                'changes.*.idproduct' => ['required_with:changes', 'integer', 'min:1'],
                'changes.*.amount'    => [
                    'required_with:changes',
                    'numeric',
                    function ($attribute, $value, $fail) use ($params) {
                        $index = explode('.', $attribute)[1]; // Obtener el índice del array changes
                        $detail = DetailTicket::find($params['changes'][$index]['id'] ?? null);

                        if (!$detail) {
                            return $fail("El detalle del ticket no es válido.");
                        }

                        if ($value > $detail->amount) {
                            return $fail("El monto ingresado en {$attribute} no puede ser mayor a {$detail->amount}.");
                        }

                        if ($value < 0) {
                            return $fail("El monto ingresado en {$attribute} no puede ser menor a 0.");
                        }
                    }
                ],
                'idcompany' => ['required', 'integer']
            ]);

            if ($validator->fails()) {
                throw new HttpResponseException($this->errorResponse($validator->errors()->first(), 400));
            }

            # Validar usuario ERP
            $userErp = UserERP::where('iduser', Auth::user()->id)->first();
            if (!$userErp) return $this->errorResponse('El usuario no cuenta con integración ERP.', 400);

            # Validar ticket
            $ticket = Ticket::with('customer')->find($id);
            if (!$ticket) return $this->errorResponse('Ticket seleccionado no válido.', 400);

            # Validar configuración
            $settingBilling = SettingBilling::where('idcompany', $params['idcompany'])
                ->where('documenttype', $ticket->idtype_document)
                ->first();

            if (!$settingBilling) return $this->errorResponse('La empresa no cuenta con configuración por empresa.', 400);

            # Preparar detalles para la API
            $store = Store::where('idcompany', $ticket->idcompany)->where('status', 1)->first();
            if (!$store) return $this->errorResponse('La bodega no es válida', 400);

            $detailJson = [];

            $details = DetailTicket::where('idticket', $ticket->id)->get();

            foreach ($details as $detail) {
                // Buscar coincidencia en $params['changes']
                $match = collect($params['changes'])->firstWhere('id', $detail->id);

                // Calcular diferencia si hay coincidencia, de lo contrario, cantidad es 0
                $difference = $match ? ($detail->amount - $match['amount']) : 0;

                $detailJson[] = [
                    "Codigo"        => $detail->id,
                    "Cantidad"      => $difference,
                    "Descuento"     => 0,
                    "Precio"        => $detail->price,
                    "TipoDescuento" => "1",
                    "Afecto"        => true,
                    "Bodega"        => $store->iderp,
                    "Nombre"        => $detail->productOrigin->name ?? 'Producto sin nombre'
                ];
            }

            # Preparar cuerpo para la API
            $headers = [
                'token' => $this->softnetService->getTokenSoftnet($userErp)
            ];

            #Forma de Pago
            $wayPay = WayToPay::activeForID($ticket->idway_to_pay)->first();
            $condicionPago = $wayPay->iderp;
            $one = $wayPay->name;
            $two = $ticket->seller_rut ?? '15134573-5';

            $body = json_encode([
                [
                    "Encabezado" => [
                        "Receptor"      => $ticket['customer']['rut'],
                        "Folio"         => $ticket->folio,
                        "MontoNeto"     => $ticket->sub_total,
                        "MontoExento"   => $ticket->mnt_excent ?? 0,
                        "Descuento"     => $ticket->total_discount,
                        "TipoDocumento" => 61,
                        "AreaNegocio"   => $ticket->bussines_area,
                        "Vendedor"      => $two,
                        "CondicionPago" => $condicionPago,
                        "Observacion"   => 'obs'
                    ],
                    "Detalle"    => $detailJson,
                    "Referencia" => [
                        "Numero"    => "1",
                        "Tipo"      => $ticket->idtype_document === 1 ? "39" : "33",
                        "Folio"     => $ticket->folio,
                        "Fecha"     => $ticket->date,
                        "CodigoNC"  => "3",
                        "Razon"     => "Anulacion parcial"
                    ],
                    "Adicional" => [
                        "Uno"       => $one,
                        "Dos"       => $two,
                    ]
                ]
            ]);

            # Ejecutar API
            $client = new Client();
            $request = new Request('POST', 'https://api2.softnet.cl/notaCreditoFacturador', $headers, $body);
            $response = $client->sendAsync($request)->wait();
            $resJson = json_decode($response->getBody()->getContents(), true);

            if (!$resJson || !isset($resJson['success']) || !$resJson['success']) {
                return $this->errorResponse('Error en la API de Softnet.', 400);
            }

            # Si la API es exitosa, actualizar base de datos
            foreach ($params['changes'] as $value) {
                $detail = DetailTicket::find($value['id']);

                if ($detail) {
                    $difference = $detail->amount - $value['amount'];
                    if ($difference > 0) {
                        # Actualizar stock
                        $lastStock = StockProduct::where('idproduct', $value['idproduct'])
                            ->where('idwarehouse', $store->id)
                            ->active()
                            ->orderBy('id', 'desc')
                            ->first();

                        $newStock = $lastStock ? $lastStock->stock_system + $difference : $difference;

                        StockProduct::create([
                            'registration_date' => Carbon::now()->toDateTimeString(),
                            'last_update'       => $lastStock->registration_date ?? Carbon::now()->toDateTimeString(),
                            'idproduct'         => $value['idproduct'],
                            'idwarehouse'       => $store->id,
                            'idcompany'         => $params['idcompany'],
                            'type_movement'     => 1,
                            'initial_stock'     => $lastStock->stock_system ?? 0,
                            'difference'        => $difference,
                            'stock_system'      => $newStock,
                            'folio'             => '',
                            'type_dte'          => '61',
                            'date_dte'          => Carbon::now()->toDateTimeString(),
                            'isAdjustment'      => 0
                        ]);
                    }
                }
            }

            # Actualizar detalles
            foreach ($params['changes'] as $value) {
                $detail = DetailTicket::find($value['id']);

                if ($detail) {
                    # Calcular montos según el tipo de cálculo
                    $amount = $value['amount'];
                    $price  = $detail->price;

                    if ($settingBilling->calculationtype == "1") {
                        # Empresa trabaja con netos
                        $net_subtotal   = $amount * $price;
                        $gross_subtotal = $net_subtotal * (1 + $this->igv);
                    } else {
                        # Empresa trabaja con brutos
                        $gross_subtotal = $amount * $price;
                        $net_subtotal   = $gross_subtotal / (1 + $this->igv);
                    }

                    # Redondear montos al próximo entero superior
                    $net_subtotal = ceil($net_subtotal);
                    $gross_subtotal = ceil($gross_subtotal);

                    # Actualizar detalle
                    $detail->update([
                        'amount'         => $amount,
                        'gross_subtotal' => $gross_subtotal,
                        'net_subtotal'   => $net_subtotal,
                        'status'         => $amount == 0 ? '2' : '1'
                    ]);
                }
            }

            # Actualizamos el carrito
            $productsShoppingCart = DetailTicket::where('idticket', $id)->active();
            $grossTotalProducts = $productsShoppingCart->sum('gross_subtotal');

            # Cálculo con netos o brutos
            $sub_total = ($settingBilling->calculationtype === "1")
                ? ($grossTotalProducts / (1 + $this->igv))
                : $grossTotalProducts;

            # Aplicar descuento global
            $discountGlobalMount = ($ticket->discounttype === 2)
                ? ($sub_total * ($ticket->global_discount / 100))
                : $ticket->global_discount;

            $sub_total -= $discountGlobalMount;

            # Calcular neto
            $net_total = ($settingBilling->calculationtype === "1")
                ? $sub_total
                : ($sub_total / (1 + $this->igv));

            # Calcular impuestos y total, redondeando al siguiente entero superior
            $igv_amount = ceil($net_total * $this->igv);
            $gross_total = ceil($net_total + $igv_amount);
            $total = ceil($grossTotalProducts - $discountGlobalMount);

            # Actualizar ticket
            $ticket->update([
                'sub_total'   => ceil($sub_total),
                'net_total'   => ceil($net_total),
                'IVA'         => $igv_amount,
                'gross_total' => $gross_total,
                'total'       => $total,
            ]);

            DB::commit();
            return $this->successResponse('Anulación parcial generada con éxito.', $ticket);
        } catch (\Throwable $th) {
            DB::rollBack();
            return $this->externalError('Error durante la anulación parcial.', $th->getMessage());
        }
    }
}
